<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class SubstituicaoPratoCardapio extends \PME\Services\TeknisaCrud {
	protected $crudService;
	protected $parameterBag;
	protected $connection;
	protected $newCode;
	protected $environment;
	protected $entityManager;

  public function __construct($crudService, $parameterBag, $connection, $newCode, $environment, $entityManager) {
      parent::__construct($crudService, $parameterBag);
      $this->connection = $connection;
      $this->newCode = $newCode;
      $this->environment = $environment;
      $this->entityManager = $entityManager;
}
	
  public function confirmSubsPrato(DTO\Request\Row $request, DTO\Response $response){
    $rows = $request->getRow();
    $nrorg = $this->environment->getNrOrg();
    
    $rows['LISTACARD'] = $this->geraListaTipoCard($rows, $nrorg);
    $rows['LISTASERV'] = $this->geraListaServico($rows, $nrorg);

		if (is_array($rows['UNIDADE']) === false) $unidade = NULL;
		else $unidade = $rows['UNIDADE']['CDFILIAL'];
    
		$connection = $this->entityManager->getConnection();
		$procedure = new StoredProcedure($connection, 'SUBSTITUI_PRATO_PAE');
		$procedure->addParam(new Param('P_CDCENTRAL',    Param::PARAM_INPUT,  $rows['CENTRAL']['CDFILIAL'],           Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_LISTATPCARD',  Param::PARAM_INPUT,  $rows['LISTACARD'],                     Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_LISTASERV',    Param::PARAM_INPUT,  $rows['LISTASERV'],                     Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT,  $rows['DTDE'],                          Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT,  $rows['DTATE'],                         Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDPRATORIG',   Param::PARAM_INPUT,  $rows['PRATO_ORIGEM']['CDPRATO'],       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDFAPRATORIG', Param::PARAM_INPUT,  $rows['PRATO_ORIGEM']['CDFILAUXPRAT'],  Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDPRATDEST',   Param::PARAM_INPUT,  $rows['PRATO_DESTINO']['CDPRATO'],      Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDFAPRATDEST', Param::PARAM_INPUT,  $rows['PRATO_DESTINO']['CDFILAUXPRAT'], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDFILIESP',    Param::PARAM_INPUT,  $unidade,                               Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRLOGINTCARD', Param::PARAM_OUTPUT, NULL,                                   Param::PARAM_TYPE_STR, 10));
		
		$nrlogintcard = $procedure->execute()['P_NRLOGINTCARD'];
    
    $result = $this->retornoLogintCard($request, $response, $nrlogintcard);
    $response->addDataSet(new DataSet("result", $result));
		
		$this->deleteListaCard($rows);
		$this->deleteListaServ($rows);
		
		return $nrlogintcard;
	}
	
	public function deleteListaCard($rows) {
	  $sql = 'DELETE FROM LISTA_TIPOCARD WHERE NRLISTA = :LISTACARD';
	  
	  $params = array(
      "LISTACARD" => $rows['LISTACARD']
    );
    
    $std = $this->connection->prepare($sql);
    $std->execute($params); 
	}
	
	public function deleteListaServ($rows) { 
	  $sql = 'DELETE FROM LISTA_SERVICO WHERE NRLISTA = :LISTASERV';
	  
	  $params = array(
      "LISTASERV" => $rows['LISTASERV']
    );
    
    $std = $this->connection->prepare($sql);
    $std->execute($params); 
	}
	
	public function retornoLogintCard($request, $response, $nrlogintcard) {
	  $sql = "SELECT L.CDFILCENTPME, L.CDTIPOCARD, T.NMTIPOCARD, TO_CHAR(L.DTCARDIAPME, 'DD/MM/YYYY') AS DTCARDIAPME, L.CDSERVFILI, S.NMSERVICO, L.CDFILIAL, L.CDPRATO, L.CDFILAUXPRAT, L.CDPRODUTO, P.NMPRODUTO, L.NRERRO FROM LOGINTCARDPAE L LEFT JOIN TIPOCARD T ON T.CDFILCENTPME = L.CDFILCENTPME AND T.CDTIPOCARD = L.CDTIPOCARD LEFT JOIN SERVICO S ON S.CDFILIAL = L.CDFILCENTPME AND S.CDSERVFILI = L.CDSERVFILI LEFT JOIN PRODUTO P ON P.CDPRODUTO = L.CDPRODUTO WHERE L.NRLOGINTCARDPAE = :P_NRLOGINTCARDPAE";
    $params = array(
      "P_NRLOGINTCARDPAE" => $nrlogintcard
    );
    $stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    $result = $stmt->fetchAll();
		return $result;
	}
	
	public function geraListaTipoCard($rows, $nrorg) {
    $nrlista = $this->newCode->getCodeByTableName('LISTA_TIPOCARD', $nrorg);
    $cardapios = $rows['TIPOCARD'];
    
    if(count($cardapios) == 0){
      $cardapios = $this->buscaCardapios($rows, $nrorg);
    }
    
    $sql = "INSERT INTO LISTA_TIPOCARD (NRLISTA, CDFILCENTPME, CDTIPOCARD, NMTIPOCARD, NRORG) VALUES (:NRLISTA, :CDFILCENTPME, :CDTIPOCARD, :NMTIPOCARD, :NRORG)";
    
    foreach ($cardapios as $card) {
      $stmt = $this->entityManager->getConnection()->prepare($sql);
      $stmt->execute(
          array(
            "NRLISTA"      => $nrlista,
            "CDFILCENTPME" => $rows['CENTRAL']['CDFILIAL'],
            "CDTIPOCARD"   => $card["CDTIPOCARD"],
            "NMTIPOCARD"   => $card['NMTIPOCARD'],
            "NRORG"        => $nrorg
          )
      );
    }
    return $nrlista;
  }
  
  public function geraListaServico($rows, $nrorg) {
    $nrlista = $this->newCode->getCodeByTableName('LISTA_SERVICO', $nrorg);
    $servicosFiliais = $rows['SERVICO'];
    
    if($rows['GRUPOSERVICO'] == NULL) $rows['GRUPOSERVICO'] = 'T';
    
    if(count($servicosFiliais) == 0){
      $servicosFiliais = $this->buscaServicosCentral($rows);
    }
    
    $sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, NRORG) VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV, :NRORG)";
    
    foreach ($servicosFiliais as $servicoFilial) {
      $stmt = $this->entityManager->getConnection()->prepare($sql);
      $stmt->execute(
        array(
          "NRLISTA"    => $nrlista,
          "CDFILIAL"   => $rows['CENTRAL']['CDFILIAL'],
          "NRORG"      => $nrorg,
          "CDSERVFILI" => $servicoFilial['CDSERVFILI'],
          "NMSERVICO"  => $servicoFilial['NMSERVICO'],
          "CDGRUPSERV" => $rows['GRUPOSERVICO']
        )
      );
    }
    return $nrlista;
  }
  
  public function buscaServicosCentral($rows) {
    $sql = "SELECT S.CDGRUPSERV, S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO, S.NRORG FROM SERVICO S WHERE ( S.CDFILIAL = :P_CDFILIAL ) AND ( S.CDGRUPSERV = :P_CDGRUPSERV OR 'T' = :P_CDGRUPSERV ) ORDER BY S.CDFILIAL, S.CDSERVFILI";
          
    $params = array(
      "P_CDGRUPSERV" => $rows['GRUPOSERVICO'],
      "P_CDFILIAL" => $rows['CENTRAL']['CDFILIAL']
    );
    
    $stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    $result = $stmt->fetchAll();
    
		return $result;
  }
  
  public function buscaCardapios($rows, $nrorg) {
    $sql = "SELECT CDTIPOCARD, NMTIPOCARD, (CDTIPOCARD || ' - ' || NMTIPOCARD) AS DESCRICAO,  IDTPCARDATIV, NRPASTATCARD, NRORG FROM TIPOCARD WHERE NRORG = :NRORG AND CDFILCENTPME = :CDFILCENTPME AND (NRPASTATCARD = :NRPASTATCARD OR :NRPASTATCARD IS NULL) ORDER BY CDTIPOCARD";
          
    $params = array(
      "NRPASTATCARD" => null,
      "CDFILCENTPME" => $rows['CENTRAL']['CDFILIAL'],
      "NRORG"        => $nrorg
    );
    
    $stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    $result = $stmt->fetchAll();
    
		return $result;
  }
}